@@ -17,22 +17,22 @@
 
 #include "mozilla/Attributes.h"
 #include "mozilla/gfx/gfxVars.h"
-#include "mozilla/layers/AsyncCompositionManager.h"  // for ViewTransform
 #include "mozilla/layers/GeckoContentController.h"
 #include "mozilla/layers/CompositorBridgeParent.h"
-#include "mozilla/layers/LayerMetricsWrapper.h"
 #include "mozilla/layers/APZThreadUtils.h"
 #include "mozilla/layers/MatrixMessage.h"
+#include "mozilla/StaticPrefs_layout.h"
 #include "mozilla/TypedEnumBits.h"
 #include "mozilla/UniquePtr.h"
 #include "apz/src/APZCTreeManager.h"
 #include "apz/src/AsyncPanZoomController.h"
 #include "apz/src/HitTestingTreeNode.h"
 #include "base/task.h"
+#include "gfxPlatform.h"
 #include "Layers.h"
 #include "TestLayers.h"
+#include "TestWRScrollData.h"
 #include "UnitTransforms.h"
-#include "gfxPrefs.h"
 
 using namespace mozilla;
 using namespace mozilla::gfx;
@@ -45,6 +45,15 @@ using ::testing::MockFunction;
 using ::testing::NiceMock;
 typedef mozilla::layers::GeckoContentController::TapType TapType;
 
+static TimeStamp GetStartupTime() {
+  static TimeStamp sStartupTime = TimeStamp::Now();
+  return sStartupTime;
+}
+
+inline uint32_t MillisecondsSinceStartup(TimeStamp aTime) {
+  return (aTime - GetStartupTime()).ToMilliseconds();
+}
+
 // Some helper functions for constructing input event objects suitable to be
 // passed either to an APZC (which expects an transformed point), or to an APZTM
 // (which expects an untransformed point). We handle both cases by setting both
@@ -67,16 +76,17 @@ inline PinchGestureInput CreatePinchGestureInput(
     PinchGestureInput::PinchGestureType aType, const ScreenPoint& aFocus,
     float aCurrentSpan, float aPreviousSpan, TimeStamp timestamp) {
   ParentLayerPoint localFocus(aFocus.x, aFocus.y);
-  PinchGestureInput result(aType, 0, timestamp, ExternalPoint(0, 0), aFocus,
-                           aCurrentSpan, aPreviousSpan, 0);
+  PinchGestureInput result(
+      aType, PinchGestureInput::UNKNOWN, MillisecondsSinceStartup(timestamp),
+      timestamp, ExternalPoint(0, 0), aFocus, aCurrentSpan, aPreviousSpan, 0);
   return result;
 }
 
 template <class SetArg, class Storage>
 class ScopedGfxSetting {
  public:
-  ScopedGfxSetting(SetArg (*aGetPrefFunc)(void), void (*aSetPrefFunc)(SetArg),
-                   SetArg aVal)
+  ScopedGfxSetting(const std::function<SetArg(void)>& aGetPrefFunc,
+                   const std::function<void(SetArg)>& aSetPrefFunc, SetArg aVal)
       : mSetPrefFunc(aSetPrefFunc) {
     mOldVal = aGetPrefFunc();
     aSetPrefFunc(aVal);
@@ -85,38 +95,46 @@ class ScopedGfxSetting {
   ~ScopedGfxSetting() { mSetPrefFunc(mOldVal); }
 
  private:
-  void (*mSetPrefFunc)(SetArg);
+  std::function<void(SetArg)> mSetPrefFunc;
   Storage mOldVal;
 };
 
-#define SCOPED_GFX_PREF(prefBase, prefType, prefValue)  \
-  ScopedGfxSetting<prefType, prefType> pref_##prefBase( \
-      &(gfxPrefs::prefBase), &(gfxPrefs::Set##prefBase), prefValue)
+static inline constexpr auto kDefaultTouchBehavior =
+    AllowedTouchBehavior::VERTICAL_PAN | AllowedTouchBehavior::HORIZONTAL_PAN |
+    AllowedTouchBehavior::PINCH_ZOOM | AllowedTouchBehavior::DOUBLE_TAP_ZOOM;
 
-#define SCOPED_GFX_VAR(varBase, varType, varValue)         \
-  ScopedGfxSetting<const varType&, varType> var_##varBase( \
-      &(gfxVars::varBase), &(gfxVars::Set##varBase), varValue)
+#define FRESH_PREF_VAR_PASTE(id, line) id##line
+#define FRESH_PREF_VAR_EXPAND(id, line) FRESH_PREF_VAR_PASTE(id, line)
+#define FRESH_PREF_VAR FRESH_PREF_VAR_EXPAND(pref, __LINE__)
 
-static TimeStamp GetStartupTime() {
-  static TimeStamp sStartupTime = TimeStamp::Now();
-  return sStartupTime;
-}
+#define SCOPED_GFX_PREF_BOOL(prefName, prefValue)                           \
+  ScopedGfxSetting<bool, bool> FRESH_PREF_VAR(                              \
+      [=]() { return Preferences::GetBool(prefName); },                     \
+      [=](bool aPrefValue) { Preferences::SetBool(prefName, aPrefValue); }, \
+      prefValue)
+
+#define SCOPED_GFX_PREF_INT(prefName, prefValue)                              \
+  ScopedGfxSetting<int32_t, int32_t> FRESH_PREF_VAR(                          \
+      [=]() { return Preferences::GetInt(prefName); },                        \
+      [=](int32_t aPrefValue) { Preferences::SetInt(prefName, aPrefValue); }, \
+      prefValue)
+
+#define SCOPED_GFX_PREF_FLOAT(prefName, prefValue)                            \
+  ScopedGfxSetting<float, float> FRESH_PREF_VAR(                              \
+      [=]() { return Preferences::GetFloat(prefName); },                      \
+      [=](float aPrefValue) { Preferences::SetFloat(prefName, aPrefValue); }, \
+      prefValue)
 
 class MockContentController : public GeckoContentController {
  public:
-  MOCK_METHOD1(NotifyLayerTransforms, void(const nsTArray<MatrixMessage>&));
+  MOCK_METHOD1(NotifyLayerTransforms, void(nsTArray<MatrixMessage>&&));
   MOCK_METHOD1(RequestContentRepaint, void(const RepaintRequest&));
-  MOCK_METHOD2(RequestFlingSnap,
-               void(const ScrollableLayerGuid::ViewID& aScrollId,
-                    const mozilla::CSSPoint& aDestination));
-  MOCK_METHOD2(AcknowledgeScrollUpdate,
-               void(const ScrollableLayerGuid::ViewID&,
-                    const uint32_t& aScrollGeneration));
   MOCK_METHOD5(HandleTap, void(TapType, const LayoutDevicePoint&, Modifiers,
                                const ScrollableLayerGuid&, uint64_t));
-  MOCK_METHOD4(NotifyPinchGesture,
+  MOCK_METHOD5(NotifyPinchGesture,
                void(PinchGestureInput::PinchGestureType,
-                    const ScrollableLayerGuid&, LayoutDeviceCoord, Modifiers));
+                    const ScrollableLayerGuid&, const LayoutDevicePoint&,
+                    LayoutDeviceCoord, Modifiers));
   // Can't use the macros with already_AddRefed :(
   void PostDelayedTask(already_AddRefed<Runnable> aTask, int aDelayMs) {
     RefPtr<Runnable> task = aTask;
@@ -140,16 +158,18 @@ class MockContentController : public GeckoContentController {
 
 class MockContentControllerDelayed : public MockContentController {
  public:
-  MockContentControllerDelayed() : mTime(GetStartupTime()) {}
+  MockContentControllerDelayed()
+      : mTime(SampleTime::FromTest(GetStartupTime())) {}
 
-  const TimeStamp& Time() { return mTime; }
+  const TimeStamp& Time() { return mTime.Time(); }
+  const SampleTime& GetSampleTime() { return mTime; }
 
   void AdvanceByMillis(int aMillis) {
     AdvanceBy(TimeDuration::FromMilliseconds(aMillis));
   }
 
   void AdvanceBy(const TimeDuration& aIncrement) {
-    TimeStamp target = mTime + aIncrement;
+    SampleTime target = mTime + aIncrement;
     while (mTaskQueue.Length() > 0 && mTaskQueue[0].second <= target) {
       RunNextDelayedTask();
     }
@@ -158,7 +178,7 @@ class MockContentControllerDelayed : public MockContentController {
 
   void PostDelayedTask(already_AddRefed<Runnable> aTask, int aDelayMs) {
     RefPtr<Runnable> task = aTask;
-    TimeStamp runAtTime = mTime + TimeDuration::FromMilliseconds(aDelayMs);
+    SampleTime runAtTime = mTime + TimeDuration::FromMilliseconds(aDelayMs);
     int insIndex = mTaskQueue.Length();
     while (insIndex > 0) {
       if (mTaskQueue[insIndex - 1].second <= runAtTime) {
@@ -175,8 +195,8 @@ class MockContentControllerDelayed : public MockContentController {
   // in the queue after this function is called. Only when the return
   // value is 0 is the queue guaranteed to be empty.
   int RunThroughDelayedTasks() {
-    nsTArray<std::pair<RefPtr<Runnable>, TimeStamp>> runQueue;
-    runQueue.SwapElements(mTaskQueue);
+    nsTArray<std::pair<RefPtr<Runnable>, SampleTime>> runQueue =
+        std::move(mTaskQueue);
     int numTasks = runQueue.Length();
     for (int i = 0; i < numTasks; i++) {
       mTime = runQueue[i].second;
@@ -191,7 +211,7 @@ class MockContentControllerDelayed : public MockContentController {
 
  private:
   void RunNextDelayedTask() {
-    std::pair<RefPtr<Runnable>, TimeStamp> next = mTaskQueue[0];
+    std::pair<RefPtr<Runnable>, SampleTime> next = mTaskQueue[0];
     mTaskQueue.RemoveElementAt(0);
     mTime = next.second;
     next.first->Run();
@@ -202,14 +222,15 @@ class MockContentControllerDelayed : public MockContentController {
 
   // The following array is sorted by timestamp (tasks are inserted in order by
   // timestamp).
-  nsTArray<std::pair<RefPtr<Runnable>, TimeStamp>> mTaskQueue;
-  TimeStamp mTime;
+  nsTArray<std::pair<RefPtr<Runnable>, SampleTime>> mTaskQueue;
+  SampleTime mTime;
 };
 
 class TestAPZCTreeManager : public APZCTreeManager {
  public:
-  explicit TestAPZCTreeManager(MockContentControllerDelayed* aMcc)
-      : APZCTreeManager(LayersId{0}), mcc(aMcc) {}
+  explicit TestAPZCTreeManager(MockContentControllerDelayed* aMcc,
+                               UniquePtr<IAPZHitTester> aHitTester = nullptr)
+      : APZCTreeManager(LayersId{0}, std::move(aHitTester)), mcc(aMcc) {}
 
   RefPtr<InputQueue> GetInputQueue() const { return mInputQueue; }
 
@@ -221,23 +242,11 @@ class TestAPZCTreeManager : public APZCTreeManager {
    **/
   void CancelAnimation() { EXPECT_TRUE(false); }
 
-  using APZCTreeManager::SetTargetAPZC;  // silence clang warning about overload
-  void SetTargetAPZC(uint64_t aInputBlockId,
-                     const nsTArray<ScrollableLayerGuid>& aTargets) {
-    nsTArray<SLGuidAndRenderRoot> wrapped;
-    for (const ScrollableLayerGuid& target : aTargets) {
-      wrapped.AppendElement(
-          SLGuidAndRenderRoot(target, wr::RenderRoot::Default));
-    }
-    this->SetTargetAPZC(aInputBlockId, wrapped);
-  }
-
  protected:
-  AsyncPanZoomController* NewAPZCInstance(LayersId aLayersId,
-                                          GeckoContentController* aController,
-                                          wr::RenderRoot aRenderRoot) override;
+  AsyncPanZoomController* NewAPZCInstance(
+      LayersId aLayersId, GeckoContentController* aController) override;
 
-  TimeStamp GetFrameTime() override { return mcc->Time(); }
+  SampleTime GetFrameTime() override { return mcc->GetSampleTime(); }
 
  private:
   RefPtr<MockContentControllerDelayed> mcc;
@@ -248,29 +257,22 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
   TestAsyncPanZoomController(LayersId aLayersId,
                              MockContentControllerDelayed* aMcc,
                              TestAPZCTreeManager* aTreeManager,
-                             wr::RenderRoot aRenderRoot,
                              GestureBehavior aBehavior = DEFAULT_GESTURES)
       : AsyncPanZoomController(aLayersId, aTreeManager,
-                               aTreeManager->GetInputQueue(), aMcc, aRenderRoot,
-                               aBehavior),
+                               aTreeManager->GetInputQueue(), aMcc, aBehavior),
         mWaitForMainThread(false),
         mcc(aMcc) {}
 
-  nsEventStatus ReceiveInputEvent(const InputData& aEvent,
-                                  ScrollableLayerGuid* aDummy,
-                                  uint64_t* aOutInputBlockId) {
+  APZEventResult ReceiveInputEvent(
+      InputData& aEvent,
+      const Maybe<nsTArray<uint32_t>>& aTouchBehaviors = Nothing()) {
     // This is a function whose signature matches exactly the ReceiveInputEvent
     // on APZCTreeManager. This allows us to templates for functions like
     // TouchDown, TouchUp, etc so that we can reuse the code for dispatching
     // events into both APZC and APZCTM.
-    return ReceiveInputEvent(aEvent, aOutInputBlockId);
-  }
-
-  nsEventStatus ReceiveInputEvent(const InputData& aEvent,
-                                  uint64_t* aOutInputBlockId) {
     return GetInputQueue()->ReceiveInputEvent(
         this, TargetConfirmationFlags{!mWaitForMainThread}, aEvent,
-        aOutInputBlockId);
+        aTouchBehaviors);
   }
 
   void ContentReceivedInputBlock(uint64_t aInputBlockId, bool aPreventDefault) {
@@ -292,6 +294,11 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     Metrics() = metrics;
   }
 
+  void SetScrollMetadata(const ScrollMetadata& aMetadata) {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    mScrollMetadata = aMetadata;
+  }
+
   FrameMetrics& GetFrameMetrics() {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
     return mScrollMetadata.GetMetrics();
@@ -307,6 +314,7 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     return mScrollMetadata.GetMetrics();
   }
 
+  using AsyncPanZoomController::GetOverscrollAmount;
   using AsyncPanZoomController::GetVelocityVector;
 
   void AssertStateIsReset() const {
@@ -324,26 +332,63 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
     EXPECT_EQ(SMOOTH_SCROLL, mState);
   }
 
-  void AssertNotAxisLocked() const {
+  void AssertStateIsSmoothMsdScroll() const {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(SMOOTHMSD_SCROLL, mState);
+  }
+
+  void AssertStateIsPanningLockedY() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(PANNING_LOCKED_Y, mState);
+  }
+
+  void AssertStateIsPanningLockedX() {
+    RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(PANNING_LOCKED_X, mState);
+  }
+
+  void AssertStateIsPanning() {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
     EXPECT_EQ(PANNING, mState);
   }
 
-  void AssertAxisLocked(ScrollDirection aDirection) const {
+  void AssertStateIsPanMomentum() {
     RecursiveMutexAutoLock lock(mRecursiveMutex);
+    EXPECT_EQ(PAN_MOMENTUM, mState);
+  }
+
+  void SetAxisLocked(ScrollDirections aDirections, bool aLockValue) {
+    if (aDirections.contains(ScrollDirection::eVertical)) {
+      mY.SetAxisLocked(aLockValue);
+    }
+    if (aDirections.contains(ScrollDirection::eHorizontal)) {
+      mX.SetAxisLocked(aLockValue);
+    }
+  }
+
+  void AssertNotAxisLocked() const {
+    EXPECT_FALSE(mY.IsAxisLocked());
+    EXPECT_FALSE(mX.IsAxisLocked());
+  }
+
+  void AssertAxisLocked(ScrollDirection aDirection) const {
     switch (aDirection) {
       case ScrollDirection::eHorizontal:
-        EXPECT_EQ(PANNING_LOCKED_X, mState);
+        EXPECT_TRUE(mY.IsAxisLocked());
+        EXPECT_FALSE(mX.IsAxisLocked());
         break;
       case ScrollDirection::eVertical:
-        EXPECT_EQ(PANNING_LOCKED_Y, mState);
+        EXPECT_TRUE(mX.IsAxisLocked());
+        EXPECT_FALSE(mY.IsAxisLocked());
         break;
+      default:
+        FAIL() << "input direction must be either vertical or horizontal";
     }
   }
 
   void AdvanceAnimationsUntilEnd(
       const TimeDuration& aIncrement = TimeDuration::FromMilliseconds(10)) {
-    while (AdvanceAnimations(mcc->Time())) {
+    while (AdvanceAnimations(mcc->GetSampleTime())) {
       mcc->AdvanceBy(aIncrement);
     }
   }
@@ -352,7 +397,7 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
       AsyncTransform* aOutTransform, ParentLayerPoint& aScrollOffset,
       const TimeDuration& aIncrement = TimeDuration::FromMilliseconds(0)) {
     mcc->AdvanceBy(aIncrement);
-    bool ret = AdvanceAnimations(mcc->Time());
+    bool ret = AdvanceAnimations(mcc->GetSampleTime());
     if (aOutTransform) {
       *aOutTransform =
           GetCurrentAsyncTransform(AsyncPanZoomController::eForHitTesting);
@@ -363,12 +408,17 @@ class TestAsyncPanZoomController : public AsyncPanZoomController {
   }
 
   CSSPoint GetCompositedScrollOffset() const {
-    return GetCurrentAsyncScrollOffset(AsyncPanZoomController::eForCompositing)
-        / GetFrameMetrics().GetZoom();
+    return GetCurrentAsyncScrollOffset(
+               AsyncPanZoomController::eForCompositing) /
+           GetFrameMetrics().GetZoom();
   }
 
   void SetWaitForMainThread() { mWaitForMainThread = true; }
 
+  bool IsOverscrollAnimationRunning() const {
+    return mState == PanZoomState::OVERSCROLL_ANIMATION;
+  }
+
  private:
   bool mWaitForMainThread;
   MockContentControllerDelayed* mcc;
@@ -378,6 +428,8 @@ class APZCTesterBase : public ::testing::Test {
  public:
   APZCTesterBase() { mcc = new NiceMock<MockContentControllerDelayed>(); }
 
+  virtual void SetUp() { gfxPlatform::GetPlatform(); }
+
   enum class PanOptions {
     None = 0,
     KeepFingerDown = 0x1,
@@ -405,10 +457,10 @@ class APZCTesterBase : public ::testing::Test {
   };
 
   template <class InputReceiver>
-  void Tap(const RefPtr<InputReceiver>& aTarget, const ScreenIntPoint& aPoint,
-           TimeDuration aTapLength,
-           nsEventStatus (*aOutEventStatuses)[2] = nullptr,
-           uint64_t* aOutInputBlockId = nullptr);
+  APZEventResult Tap(const RefPtr<InputReceiver>& aTarget,
+                     const ScreenIntPoint& aPoint, TimeDuration aTapLength,
+                     nsEventStatus (*aOutEventStatuses)[2] = nullptr,
+                     uint64_t* aOutInputBlockId = nullptr);
 
   template <class InputReceiver>
   void TapAndCheckStatus(const RefPtr<InputReceiver>& aTarget,
@@ -500,35 +552,31 @@ MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(APZCTesterBase::PanOptions)
 MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(APZCTesterBase::PinchOptions)
 
 template <class InputReceiver>
-void APZCTesterBase::Tap(const RefPtr<InputReceiver>& aTarget,
-                         const ScreenIntPoint& aPoint, TimeDuration aTapLength,
-                         nsEventStatus (*aOutEventStatuses)[2],
-                         uint64_t* aOutInputBlockId) {
-  // Even if the caller doesn't care about the block id, we need it to set the
-  // allowed touch behaviour below, so make sure aOutInputBlockId is non-null.
-  uint64_t blockId;
-  if (!aOutInputBlockId) {
-    aOutInputBlockId = &blockId;
-  }
-
-  nsEventStatus status =
-      TouchDown(aTarget, aPoint, mcc->Time(), aOutInputBlockId);
+APZEventResult APZCTesterBase::Tap(const RefPtr<InputReceiver>& aTarget,
+                                   const ScreenIntPoint& aPoint,
+                                   TimeDuration aTapLength,
+                                   nsEventStatus (*aOutEventStatuses)[2],
+                                   uint64_t* aOutInputBlockId) {
+  APZEventResult touchDownResult = TouchDown(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = status;
+    (*aOutEventStatuses)[0] = touchDownResult.GetStatus();
+  }
+  if (aOutInputBlockId) {
+    *aOutInputBlockId = touchDownResult.mInputBlockId;
   }
   mcc->AdvanceBy(aTapLength);
 
   // If touch-action is enabled then simulate the allowed touch behaviour
   // notification that the main thread is supposed to deliver.
-  if (gfxPrefs::TouchActionEnabled() &&
-      status != nsEventStatus_eConsumeNoDefault) {
-    SetDefaultAllowedTouchBehavior(aTarget, *aOutInputBlockId);
+  if (touchDownResult.GetStatus() != nsEventStatus_eConsumeNoDefault) {
+    SetDefaultAllowedTouchBehavior(aTarget, touchDownResult.mInputBlockId);
   }
 
-  status = TouchUp(aTarget, aPoint, mcc->Time());
+  APZEventResult touchUpResult = TouchUp(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = status;
+    (*aOutEventStatuses)[1] = touchUpResult.GetStatus();
   }
+  return touchDownResult;
 }
 
 template <class InputReceiver>
@@ -552,8 +600,8 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
   // We can't use a scoped pref because this value might be read at some later
   // time when the events are actually processed, rather than when we deliver
   // them.
-  gfxPrefs::SetAPZTouchStartTolerance(1.0f / 1000.0f);
-  gfxPrefs::SetAPZTouchMoveTolerance(0.0f);
+  Preferences::SetFloat("apz.touch_start_tolerance", 1.0f / 1000.0f);
+  Preferences::SetFloat("apz.touch_move_tolerance", 0.0f);
   int overcomeTouchToleranceX = 0;
   int overcomeTouchToleranceY = 0;
   if (!(aOptions & PanOptions::ExactCoordinates)) {
@@ -569,7 +617,7 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
   }
 
   const TimeDuration TIME_BETWEEN_TOUCH_EVENT =
-      TimeDuration::FromMilliseconds(50);
+      TimeDuration::FromMilliseconds(20);
 
   // Even if the caller doesn't care about the block id, we need it to set the
   // allowed touch behaviour below, so make sure aOutInputBlockId is non-null.
@@ -579,49 +627,61 @@ void APZCTesterBase::Pan(const RefPtr<InputReceiver>& aTarget,
   }
 
   // Make sure the move is large enough to not be handled as a tap
-  nsEventStatus status =
+  APZEventResult result =
       TouchDown(aTarget,
                 ScreenIntPoint(aTouchStart.x + overcomeTouchToleranceX,
                                aTouchStart.y + overcomeTouchToleranceY),
-                mcc->Time(), aOutInputBlockId);
+                mcc->Time());
+  if (aOutInputBlockId) {
+    *aOutInputBlockId = result.mInputBlockId;
+  }
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = status;
+    (*aOutEventStatuses)[0] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
 
   // Allowed touch behaviours must be set after sending touch-start.
-  if (status != nsEventStatus_eConsumeNoDefault) {
+  if (result.GetStatus() != nsEventStatus_eConsumeNoDefault) {
     if (aAllowedTouchBehaviors) {
       EXPECT_EQ(1UL, aAllowedTouchBehaviors->Length());
       aTarget->SetAllowedTouchBehavior(*aOutInputBlockId,
                                        *aAllowedTouchBehaviors);
-    } else if (gfxPrefs::TouchActionEnabled()) {
+    } else {
       SetDefaultAllowedTouchBehavior(aTarget, *aOutInputBlockId);
     }
   }
 
-  status = TouchMove(aTarget, aTouchStart, mcc->Time());
+  result = TouchMove(aTarget, aTouchStart, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = status;
+    (*aOutEventStatuses)[1] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
 
-  status = TouchMove(aTarget, aTouchEnd, mcc->Time());
+  const int numSteps = 3;
+  auto stepVector = (aTouchEnd - aTouchStart) / numSteps;
+  for (int k = 1; k < numSteps; k++) {
+    auto stepPoint = aTouchStart + stepVector * k;
+    Unused << TouchMove(aTarget, stepPoint, mcc->Time());
+
+    mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
+  }
+
+  result = TouchMove(aTarget, aTouchEnd, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = status;
+    (*aOutEventStatuses)[2] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
 
   if (!(aOptions & PanOptions::KeepFingerDown)) {
-    status = TouchUp(aTarget, aTouchEnd, mcc->Time());
+    result = TouchUp(aTarget, aTouchEnd, mcc->Time());
   } else {
-    status = nsEventStatus_eIgnore;
+    result.SetStatusAsIgnore();
   }
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[3] = status;
+    (*aOutEventStatuses)[3] = result.GetStatus();
   }
 
   if ((aOptions & PanOptions::NoFling)) {
@@ -669,45 +729,42 @@ void APZCTesterBase::DoubleTap(const RefPtr<InputReceiver>& aTarget,
                                const ScreenIntPoint& aPoint,
                                nsEventStatus (*aOutEventStatuses)[4],
                                uint64_t (*aOutInputBlockIds)[2]) {
-  uint64_t blockId;
-  nsEventStatus status = TouchDown(aTarget, aPoint, mcc->Time(), &blockId);
+  APZEventResult result = TouchDown(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = status;
+    (*aOutEventStatuses)[0] = result.GetStatus();
   }
   if (aOutInputBlockIds) {
-    (*aOutInputBlockIds)[0] = blockId;
+    (*aOutInputBlockIds)[0] = result.mInputBlockId;
   }
   mcc->AdvanceByMillis(10);
 
   // If touch-action is enabled then simulate the allowed touch behaviour
   // notification that the main thread is supposed to deliver.
-  if (gfxPrefs::TouchActionEnabled() &&
-      status != nsEventStatus_eConsumeNoDefault) {
-    SetDefaultAllowedTouchBehavior(aTarget, blockId);
+  if (result.GetStatus() != nsEventStatus_eConsumeNoDefault) {
+    SetDefaultAllowedTouchBehavior(aTarget, result.mInputBlockId);
   }
 
-  status = TouchUp(aTarget, aPoint, mcc->Time());
+  result = TouchUp(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = status;
+    (*aOutEventStatuses)[1] = result.GetStatus();
   }
   mcc->AdvanceByMillis(10);
-  status = TouchDown(aTarget, aPoint, mcc->Time(), &blockId);
+  result = TouchDown(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = status;
+    (*aOutEventStatuses)[2] = result.GetStatus();
   }
   if (aOutInputBlockIds) {
-    (*aOutInputBlockIds)[1] = blockId;
+    (*aOutInputBlockIds)[1] = result.mInputBlockId;
   }
   mcc->AdvanceByMillis(10);
 
-  if (gfxPrefs::TouchActionEnabled() &&
-      status != nsEventStatus_eConsumeNoDefault) {
-    SetDefaultAllowedTouchBehavior(aTarget, blockId);
+  if (result.GetStatus() != nsEventStatus_eConsumeNoDefault) {
+    SetDefaultAllowedTouchBehavior(aTarget, result.mInputBlockId);
   }
 
-  status = TouchUp(aTarget, aPoint, mcc->Time());
+  result = TouchUp(aTarget, aPoint, mcc->Time());
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[3] = status;
+    (*aOutEventStatuses)[3] = result.GetStatus();
   }
 }
 
@@ -756,15 +813,19 @@ void APZCTesterBase::PinchWithTouchInput(
   }
 
   const TimeDuration TIME_BETWEEN_TOUCH_EVENT =
-      TimeDuration::FromMilliseconds(50);
+      TimeDuration::FromMilliseconds(20);
 
   MultiTouchInput mtiStart =
       MultiTouchInput(MultiTouchInput::MULTITOUCH_START, 0, mcc->Time(), 0);
   mtiStart.mTouches.AppendElement(CreateSingleTouchData(inputId, aFocus));
   mtiStart.mTouches.AppendElement(CreateSingleTouchData(inputId + 1, aFocus));
-  nsEventStatus status = aTarget->ReceiveInputEvent(mtiStart, aOutInputBlockId);
+  APZEventResult result;
+  result = aTarget->ReceiveInputEvent(mtiStart);
+  if (aOutInputBlockId) {
+    *aOutInputBlockId = result.mInputBlockId;
+  }
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = status;
+    (*aOutEventStatuses)[0] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
@@ -773,32 +834,60 @@ void APZCTesterBase::PinchWithTouchInput(
     EXPECT_EQ(2UL, aAllowedTouchBehaviors->Length());
     aTarget->SetAllowedTouchBehavior(*aOutInputBlockId,
                                      *aAllowedTouchBehaviors);
-  } else if (gfxPrefs::TouchActionEnabled()) {
+  } else {
     SetDefaultAllowedTouchBehavior(aTarget, *aOutInputBlockId, 2);
   }
 
+  ScreenIntPoint pinchStartPoint1(aFocus.x - int32_t(pinchLength), aFocus.y);
+  ScreenIntPoint pinchStartPoint2(aFocus.x + int32_t(pinchLength), aFocus.y);
+
   MultiTouchInput mtiMove1 =
       MultiTouchInput(MultiTouchInput::MULTITOUCH_MOVE, 0, mcc->Time(), 0);
   mtiMove1.mTouches.AppendElement(
-      CreateSingleTouchData(inputId, aFocus.x - pinchLength, aFocus.y));
+      CreateSingleTouchData(inputId, pinchStartPoint1));
   mtiMove1.mTouches.AppendElement(
-      CreateSingleTouchData(inputId + 1, aFocus.x + pinchLength, aFocus.y));
-  status = aTarget->ReceiveInputEvent(mtiMove1, nullptr);
+      CreateSingleTouchData(inputId + 1, pinchStartPoint2));
+  result = aTarget->ReceiveInputEvent(mtiMove1);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = status;
+    (*aOutEventStatuses)[1] = result.GetStatus();
   }
 
   mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
 
+  // Pinch instantly but move in steps.
+  const int numSteps = 3;
+  auto stepVector = (aSecondFocus - aFocus) / numSteps;
+  for (int k = 1; k < numSteps; k++) {
+    ScreenIntPoint stepFocus = aFocus + stepVector * k;
+    ScreenIntPoint stepPoint1(stepFocus.x - int32_t(pinchLengthScaled),
+                              stepFocus.y);
+    ScreenIntPoint stepPoint2(stepFocus.x + int32_t(pinchLengthScaled),
+                              stepFocus.y);
+    MultiTouchInput mtiMoveStep =
+        MultiTouchInput(MultiTouchInput::MULTITOUCH_MOVE, 0, mcc->Time(), 0);
+    mtiMoveStep.mTouches.AppendElement(
+        CreateSingleTouchData(inputId, stepPoint1));
+    mtiMoveStep.mTouches.AppendElement(
+        CreateSingleTouchData(inputId + 1, stepPoint2));
+    Unused << aTarget->ReceiveInputEvent(mtiMoveStep);
+
+    mcc->AdvanceBy(TIME_BETWEEN_TOUCH_EVENT);
+  }
+
+  ScreenIntPoint pinchEndPoint1(aSecondFocus.x - int32_t(pinchLengthScaled),
+                                aSecondFocus.y);
+  ScreenIntPoint pinchEndPoint2(aSecondFocus.x + int32_t(pinchLengthScaled),
+                                aSecondFocus.y);
+
   MultiTouchInput mtiMove2 =
       MultiTouchInput(MultiTouchInput::MULTITOUCH_MOVE, 0, mcc->Time(), 0);
-  mtiMove2.mTouches.AppendElement(CreateSingleTouchData(
-      inputId, aSecondFocus.x - pinchLengthScaled, aSecondFocus.y));
-  mtiMove2.mTouches.AppendElement(CreateSingleTouchData(
-      inputId + 1, aSecondFocus.x + pinchLengthScaled, aSecondFocus.y));
-  status = aTarget->ReceiveInputEvent(mtiMove2, nullptr);
+  mtiMove2.mTouches.AppendElement(
+      CreateSingleTouchData(inputId, pinchEndPoint1));
+  mtiMove2.mTouches.AppendElement(
+      CreateSingleTouchData(inputId + 1, pinchEndPoint2));
+  result = aTarget->ReceiveInputEvent(mtiMove2);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = status;
+    (*aOutEventStatuses)[2] = result.GetStatus();
   }
 
   if (aOptions & (PinchOptions::LiftFinger1 | PinchOptions::LiftFinger2)) {
@@ -807,16 +896,16 @@ void APZCTesterBase::PinchWithTouchInput(
     MultiTouchInput mtiEnd =
         MultiTouchInput(MultiTouchInput::MULTITOUCH_END, 0, mcc->Time(), 0);
     if (aOptions & PinchOptions::LiftFinger1) {
-      mtiEnd.mTouches.AppendElement(CreateSingleTouchData(
-          inputId, aSecondFocus.x - pinchLengthScaled, aSecondFocus.y));
+      mtiEnd.mTouches.AppendElement(
+          CreateSingleTouchData(inputId, pinchEndPoint1));
     }
     if (aOptions & PinchOptions::LiftFinger2) {
-      mtiEnd.mTouches.AppendElement(CreateSingleTouchData(
-          inputId + 1, aSecondFocus.x + pinchLengthScaled, aSecondFocus.y));
+      mtiEnd.mTouches.AppendElement(
+          CreateSingleTouchData(inputId + 1, pinchEndPoint2));
     }
-    status = aTarget->ReceiveInputEvent(mtiEnd, nullptr);
+    result = aTarget->ReceiveInputEvent(mtiEnd);
     if (aOutEventStatuses) {
-      (*aOutEventStatuses)[3] = status;
+      (*aOutEventStatuses)[3] = result.GetStatus();
     }
   }
 
@@ -848,32 +937,29 @@ void APZCTesterBase::PinchWithPinchInput(
   const TimeDuration TIME_BETWEEN_PINCH_INPUT =
       TimeDuration::FromMilliseconds(50);
 
-  nsEventStatus actualStatus = aTarget->ReceiveInputEvent(
-      CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_START, aFocus,
-                              10.0, 10.0, mcc->Time()),
-      nullptr);
+  auto event = CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_START,
+                                       aFocus, 10.0, 10.0, mcc->Time());
+  APZEventResult actual = aTarget->ReceiveInputEvent(event);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[0] = actualStatus;
+    (*aOutEventStatuses)[0] = actual.GetStatus();
   }
   mcc->AdvanceBy(TIME_BETWEEN_PINCH_INPUT);
 
-  actualStatus = aTarget->ReceiveInputEvent(
+  event =
       CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_SCALE,
-                              aSecondFocus, 10.0 * aScale, 10.0, mcc->Time()),
-      nullptr);
+                              aSecondFocus, 10.0 * aScale, 10.0, mcc->Time());
+  actual = aTarget->ReceiveInputEvent(event);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[1] = actualStatus;
+    (*aOutEventStatuses)[1] = actual.GetStatus();
   }
   mcc->AdvanceBy(TIME_BETWEEN_PINCH_INPUT);
 
-  actualStatus = aTarget->ReceiveInputEvent(
-      CreatePinchGestureInput(
-          PinchGestureInput::PINCHGESTURE_END,
-          PinchGestureInput::BothFingersLifted<ScreenPixel>(), 10.0 * aScale,
-          10.0 * aScale, mcc->Time()),
-      nullptr);
+  event =
+      CreatePinchGestureInput(PinchGestureInput::PINCHGESTURE_END, aSecondFocus,
+                              10.0 * aScale, 10.0 * aScale, mcc->Time());
+  actual = aTarget->ReceiveInputEvent(event);
   if (aOutEventStatuses) {
-    (*aOutEventStatuses)[2] = actualStatus;
+    (*aOutEventStatuses)[2] = actual.GetStatus();
   }
 }
 
@@ -885,35 +971,20 @@ void APZCTesterBase::PinchWithPinchInputAndCheckStatus(
   PinchWithPinchInput(aTarget, aFocus, aFocus, aScale, &statuses);
 
   nsEventStatus expectedStatus = aShouldTriggerPinch
-                                     ? nsEventStatus_eConsumeNoDefault
+                                     ? nsEventStatus_eConsumeDoDefault
                                      : nsEventStatus_eIgnore;
   EXPECT_EQ(expectedStatus, statuses[0]);
   EXPECT_EQ(expectedStatus, statuses[1]);
 }
 
-AsyncPanZoomController* TestAPZCTreeManager::NewAPZCInstance(
-    LayersId aLayersId, GeckoContentController* aController,
-    wr::RenderRoot aRenderRoot) {
-  MockContentControllerDelayed* mcc =
-      static_cast<MockContentControllerDelayed*>(aController);
-  return new TestAsyncPanZoomController(
-      aLayersId, mcc, this, aRenderRoot,
-      AsyncPanZoomController::USE_GESTURE_DETECTOR);
-}
-
 inline FrameMetrics TestFrameMetrics() {
   FrameMetrics fm;
 
   fm.SetDisplayPort(CSSRect(0, 0, 10, 10));
   fm.SetCompositionBounds(ParentLayerRect(0, 0, 10, 10));
-  fm.SetCriticalDisplayPort(CSSRect(0, 0, 10, 10));
   fm.SetScrollableRect(CSSRect(0, 0, 100, 100));
 
   return fm;
 }
 
-inline uint32_t MillisecondsSinceStartup(TimeStamp aTime) {
-  return (aTime - GetStartupTime()).ToMilliseconds();
-}
-
 #endif  // mozilla_layers_APZTestCommon_h
