@@ -11,7 +11,6 @@
 #include "mozilla/layers/CompositorBridgeChild.h"
 #include "SharedSurfaceGL.h"
 #include "WebRenderBridgeChild.h"
-#include "RenderRootStateManager.h"
 
 namespace mozilla {
 namespace layers {
@@ -20,39 +19,27 @@ CompositableForwarder* WebRenderCanvasRenderer::GetForwarder() {
   return mManager->WrBridge();
 }
 
-void WebRenderCanvasRenderer::Initialize(const CanvasInitializeData& aData) {
-  ShareableCanvasRenderer::Initialize(aData);
-}
-
-WebRenderCanvasRendererAsync::~WebRenderCanvasRendererAsync() { Destroy(); }
-
-void WebRenderCanvasRendererAsync::Initialize(
-    const CanvasInitializeData& aData) {
-  WebRenderCanvasRenderer::Initialize(aData);
-
+WebRenderCanvasRendererAsync::~WebRenderCanvasRendererAsync() {
   if (mPipelineId.isSome()) {
     mManager->RemovePipelineIdForCompositable(mPipelineId.ref());
     mPipelineId.reset();
   }
 }
 
+void WebRenderCanvasRendererAsync::Initialize(const CanvasRendererData& aData) {
+  WebRenderCanvasRenderer::Initialize(aData);
+
+  ClearCachedResources();
+}
+
 bool WebRenderCanvasRendererAsync::CreateCompositable() {
   if (!mCanvasClient) {
-    TextureFlags flags = TextureFlags::DEFAULT;
-    if (mOriginPos == gl::OriginPos::BottomLeft) {
-      flags |= TextureFlags::ORIGIN_BOTTOM_LEFT;
+    auto compositableFlags = TextureFlags::NO_FLAGS;
+    if (!mData.mIsAlphaPremult) {
+      // WR needs this flag marked on the compositable, not just the texture.
+      compositableFlags |= TextureFlags::NON_PREMULTIPLIED;
     }
-
-    if (!mIsAlphaPremultiplied) {
-      flags |= TextureFlags::NON_PREMULTIPLIED;
-    }
-
-    mCanvasClient = CanvasClient::CreateCanvasClient(GetCanvasClientType(),
-                                                     GetForwarder(), flags);
-    if (!mCanvasClient) {
-      return false;
-    }
-
+    mCanvasClient = new CanvasClient(GetForwarder(), compositableFlags);
     mCanvasClient->Connect();
   }
 
@@ -60,8 +47,9 @@ bool WebRenderCanvasRendererAsync::CreateCompositable() {
     // Alloc async image pipeline id.
     mPipelineId = Some(
         mManager->WrBridge()->GetCompositorBridgeChild()->GetNextPipelineId());
-    mManager->AddPipelineIdForCompositable(mPipelineId.ref(),
-                                           mCanvasClient->GetIPCHandle());
+    mManager->AddPipelineIdForCompositable(
+        mPipelineId.ref(), mCanvasClient->GetIPCHandle(),
+        CompositableHandleOwner::WebRenderBridge);
   }
 
   return true;
@@ -74,17 +62,11 @@ void WebRenderCanvasRendererAsync::ClearCachedResources() {
   }
 }
 
-void WebRenderCanvasRendererAsync::Destroy() {
-  if (mPipelineId.isSome()) {
-    mManager->RemovePipelineIdForCompositable(mPipelineId.ref());
-    mPipelineId.reset();
-  }
-}
-
 void WebRenderCanvasRendererAsync::
     UpdateCompositableClientForEmptyTransaction() {
-  UpdateCompositableClient(mManager->GetRenderRoot());
-  if (mPipelineId.isSome()) {
+  bool wasDirty = IsDirty();
+  UpdateCompositableClient();
+  if (wasDirty && mPipelineId.isSome()) {
     // Notify an update of async image pipeline during empty transaction.
     // During non empty transaction, WebRenderBridgeParent receives
     // OpUpdateAsyncImagePipeline message, but during empty transaction, the
