@@ -8,9 +8,9 @@
 #ifndef SkNx_DEFINED
 #define SkNx_DEFINED
 
-#include "SkSafe_math.h"
-#include "SkScalar.h"
-#include "SkTypes.h"
+#include "include/core/SkScalar.h"
+#include "include/core/SkTypes.h"
+#include "include/private/SkSafe_math.h"
 
 #include <algorithm>
 #include <limits>
@@ -415,9 +415,9 @@ typedef SkNx<8, uint32_t> Sk8u;
 
 // Include platform specific specializations if available.
 #if !defined(SKNX_NO_SIMD) && SK_CPU_SSE_LEVEL >= SK_CPU_SSE_LEVEL_SSE2
-    #include "SkNx_sse.h"
+    #include "include/private/SkNx_sse.h"
 #elif !defined(SKNX_NO_SIMD) && defined(SK_ARM_HAS_NEON)
-    #include "SkNx_neon.h"
+    #include "include/private/SkNx_neon.h"
 #else
 
 AI static Sk4i Sk4f_round(const Sk4f& x) {
