@@ -8,9 +8,9 @@
 #ifndef GrDrawVerticesOp_DEFINED
 #define GrDrawVerticesOp_DEFINED
 
-#include "GrTypesPriv.h"
-#include "SkRefCnt.h"
-#include "SkVertices.h"
+#include "include/core/SkRefCnt.h"
+#include "include/core/SkVertices.h"
+#include "include/private/GrTypesPriv.h"
 
 class GrColorSpaceXform;
 class GrDrawOp;
