@@ -5,8 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsDeviceContext.h"
-#include <algorithm>      // for max
-#include "gfxASurface.h"  // for gfxASurface, etc
+#include <algorithm>  // for max
 #include "gfxContext.h"
 #include "gfxImageSurface.h"     // for gfxImageSurface
 #include "gfxPoint.h"            // for gfxSize
@@ -20,6 +19,7 @@
 #include "nsCRT.h"                // for nsCRT
 #include "nsDebug.h"              // for NS_ASSERTION, etc
 #include "nsFont.h"               // for nsFont
+#include "nsFontCache.h"          // for nsFontCache
 #include "nsFontMetrics.h"        // for nsFontMetrics
 #include "nsAtom.h"               // for nsAtom, NS_Atomize
 #include "nsID.h"
@@ -41,194 +41,8 @@
 
 using namespace mozilla;
 using namespace mozilla::gfx;
-using mozilla::services::GetObserverService;
 using mozilla::widget::ScreenManager;
 
-class nsFontCache final : public nsIObserver {
- public:
-  nsFontCache() : mContext(nullptr) {}
-
-  NS_DECL_THREADSAFE_ISUPPORTS
-  NS_DECL_NSIOBSERVER
-
-  void Init(nsDeviceContext* aContext);
-  void Destroy();
-
-  already_AddRefed<nsFontMetrics> GetMetricsFor(
-      const nsFont& aFont, const nsFontMetrics::Params& aParams);
-
-  void FontMetricsDeleted(const nsFontMetrics* aFontMetrics);
-  void Compact();
-
-  // Flush aFlushCount oldest entries, or all if aFlushCount is negative
-  void Flush(int32_t aFlushCount = -1);
-
-  void UpdateUserFonts(gfxUserFontSet* aUserFontSet);
-
- protected:
-  // If the array of cached entries is about to exceed this threshold,
-  // we'll discard the oldest ones so as to keep the size reasonable.
-  // In practice, the great majority of cache hits are among the last
-  // few entries; keeping thousands of older entries becomes counter-
-  // productive because it can then take too long to scan the cache.
-  static const int32_t kMaxCacheEntries = 128;
-
-  ~nsFontCache() {}
-
-  nsDeviceContext* mContext;  // owner
-  RefPtr<nsAtom> mLocaleLanguage;
-
-  // We may not flush older entries immediately the array reaches
-  // kMaxCacheEntries length, because this usually happens on a stylo
-  // thread where we can't safely delete metrics objects. So we allocate an
-  // oversized autoarray buffer here, so that we're unlikely to overflow
-  // it and need separate heap allocation before the flush happens on the
-  // main thread.
-  AutoTArray<nsFontMetrics*, kMaxCacheEntries * 2> mFontMetrics;
-
-  bool mFlushPending = false;
-
-  class FlushFontMetricsTask : public mozilla::Runnable {
-   public:
-    explicit FlushFontMetricsTask(nsFontCache* aCache)
-        : mozilla::Runnable("FlushFontMetricsTask"), mCache(aCache) {}
-    NS_IMETHOD Run() override {
-      // Partially flush the cache, leaving the kMaxCacheEntries/2 most
-      // recent entries.
-      mCache->Flush(mCache->mFontMetrics.Length() - kMaxCacheEntries / 2);
-      mCache->mFlushPending = false;
-      return NS_OK;
-    }
-
-   private:
-    RefPtr<nsFontCache> mCache;
-  };
-};
-
-NS_IMPL_ISUPPORTS(nsFontCache, nsIObserver)
-
-// The Init and Destroy methods are necessary because it's not
-// safe to call AddObserver from a constructor or RemoveObserver
-// from a destructor.  That should be fixed.
-void nsFontCache::Init(nsDeviceContext* aContext) {
-  mContext = aContext;
-  // register as a memory-pressure observer to free font resources
-  // in low-memory situations.
-  nsCOMPtr<nsIObserverService> obs = GetObserverService();
-  if (obs) obs->AddObserver(this, "memory-pressure", false);
-
-  mLocaleLanguage = nsLanguageAtomService::GetService()->GetLocaleLanguage();
-  if (!mLocaleLanguage) {
-    mLocaleLanguage = NS_Atomize("x-western");
-  }
-}
-
-void nsFontCache::Destroy() {
-  nsCOMPtr<nsIObserverService> obs = GetObserverService();
-  if (obs) obs->RemoveObserver(this, "memory-pressure");
-  Flush();
-}
-
-NS_IMETHODIMP
-nsFontCache::Observe(nsISupports*, const char* aTopic, const char16_t*) {
-  if (!nsCRT::strcmp(aTopic, "memory-pressure")) Compact();
-  return NS_OK;
-}
-
-already_AddRefed<nsFontMetrics> nsFontCache::GetMetricsFor(
-    const nsFont& aFont, const nsFontMetrics::Params& aParams) {
-  nsAtom* language =
-      aParams.language ? aParams.language : mLocaleLanguage.get();
-
-  // First check our cache
-  // start from the end, which is where we put the most-recent-used element
-  const int32_t n = mFontMetrics.Length() - 1;
-  for (int32_t i = n; i >= 0; --i) {
-    nsFontMetrics* fm = mFontMetrics[i];
-    if (fm->Font().Equals(aFont) &&
-        fm->GetUserFontSet() == aParams.userFontSet &&
-        fm->Language() == language &&
-        fm->Orientation() == aParams.orientation) {
-      if (i != n) {
-        // promote it to the end of the cache
-        mFontMetrics.RemoveElementAt(i);
-        mFontMetrics.AppendElement(fm);
-      }
-      fm->GetThebesFontGroup()->UpdateUserFonts();
-      return do_AddRef(fm);
-    }
-  }
-
-  // It's not in the cache. Get font metrics and then cache them.
-  // If the cache has reached its size limit, drop the older half of the
-  // entries; but if we're on a stylo thread (the usual case), we have
-  // to post a task back to the main thread to do the flush.
-  if (n >= kMaxCacheEntries - 1 && !mFlushPending) {
-    if (NS_IsMainThread()) {
-      Flush(mFontMetrics.Length() - kMaxCacheEntries / 2);
-    } else {
-      mFlushPending = true;
-      nsCOMPtr<nsIRunnable> flushTask = new FlushFontMetricsTask(this);
-      MOZ_ALWAYS_SUCCEEDS(NS_DispatchToMainThread(flushTask));
-    }
-  }
-
-  nsFontMetrics::Params params = aParams;
-  params.language = language;
-  RefPtr<nsFontMetrics> fm = new nsFontMetrics(aFont, params, mContext);
-  // the mFontMetrics list has the "head" at the end, because append
-  // is cheaper than insert
-  mFontMetrics.AppendElement(do_AddRef(fm).take());
-  return fm.forget();
-}
-
-void nsFontCache::UpdateUserFonts(gfxUserFontSet* aUserFontSet) {
-  for (nsFontMetrics* fm : mFontMetrics) {
-    gfxFontGroup* fg = fm->GetThebesFontGroup();
-    if (fg->GetUserFontSet() == aUserFontSet) {
-      fg->UpdateUserFonts();
-    }
-  }
-}
-
-void nsFontCache::FontMetricsDeleted(const nsFontMetrics* aFontMetrics) {
-  mFontMetrics.RemoveElement(aFontMetrics);
-}
-
-void nsFontCache::Compact() {
-  // Need to loop backward because the running element can be removed on
-  // the way
-  for (int32_t i = mFontMetrics.Length() - 1; i >= 0; --i) {
-    nsFontMetrics* fm = mFontMetrics[i];
-    nsFontMetrics* oldfm = fm;
-    // Destroy() isn't here because we want our device context to be
-    // notified
-    NS_RELEASE(fm);  // this will reset fm to nullptr
-    // if the font is really gone, it would have called back in
-    // FontMetricsDeleted() and would have removed itself
-    if (mFontMetrics.IndexOf(oldfm) != mFontMetrics.NoIndex) {
-      // nope, the font is still there, so let's hold onto it too
-      NS_ADDREF(oldfm);
-    }
-  }
-}
-
-// Flush the aFlushCount oldest entries, or all if (aFlushCount < 0)
-void nsFontCache::Flush(int32_t aFlushCount) {
-  int32_t n = aFlushCount < 0
-                  ? mFontMetrics.Length()
-                  : std::min<int32_t>(aFlushCount, mFontMetrics.Length());
-  for (int32_t i = n - 1; i >= 0; --i) {
-    nsFontMetrics* fm = mFontMetrics[i];
-    // Destroy() will unhook our device context from the fm so that we
-    // won't waste time in triggering the notification of
-    // FontMetricsDeleted() in the subsequent release
-    fm->Destroy();
-    NS_RELEASE(fm);
-  }
-  mFontMetrics.RemoveElementsAt(0, n);
-}
-
 nsDeviceContext::nsDeviceContext()
     : mWidth(0),
       mHeight(0),
@@ -247,45 +61,7 @@ nsDeviceContext::nsDeviceContext()
   MOZ_ASSERT(NS_IsMainThread(), "nsDeviceContext created off main thread");
 }
 
-nsDeviceContext::~nsDeviceContext() {
-  if (mFontCache) {
-    mFontCache->Destroy();
-  }
-}
-
-void nsDeviceContext::InitFontCache() {
-  if (!mFontCache) {
-    mFontCache = new nsFontCache();
-    mFontCache->Init(this);
-  }
-}
-
-void nsDeviceContext::UpdateFontCacheUserFonts(gfxUserFontSet* aUserFontSet) {
-  if (mFontCache) {
-    mFontCache->UpdateUserFonts(aUserFontSet);
-  }
-}
-
-already_AddRefed<nsFontMetrics> nsDeviceContext::GetMetricsFor(
-    const nsFont& aFont, const nsFontMetrics::Params& aParams) {
-  InitFontCache();
-  return mFontCache->GetMetricsFor(aFont, aParams);
-}
-
-nsresult nsDeviceContext::FlushFontCache(void) {
-  if (mFontCache) mFontCache->Flush();
-  return NS_OK;
-}
-
-nsresult nsDeviceContext::FontMetricsDeleted(
-    const nsFontMetrics* aFontMetrics) {
-  if (mFontCache) {
-    mFontCache->FontMetricsDeleted(aFontMetrics);
-  }
-  return NS_OK;
-}
-
-bool nsDeviceContext::IsPrinterContext() { return mPrintTarget != nullptr; }
+nsDeviceContext::~nsDeviceContext() = default;
 
 void nsDeviceContext::SetDPI(double* aScale) {
   float dpi;
@@ -298,17 +74,15 @@ void nsDeviceContext::SetDPI(double* aScale) {
     mAppUnitsPerDevPixelAtUnitFullZoom =
         NS_lround((AppUnitsPerCSSPixel() * 96) / dpi);
   } else {
-    nsCOMPtr<nsIScreen> primaryScreen;
-    ScreenManager& screenManager = ScreenManager::GetSingleton();
-    screenManager.GetPrimaryScreen(getter_AddRefs(primaryScreen));
+    RefPtr<widget::Screen> primaryScreen =
+        ScreenManager::GetSingleton().GetPrimaryScreen();
     MOZ_ASSERT(primaryScreen);
 
     // A value of -1 means use the maximum of 96 and the system DPI.
     // A value of 0 means use the system DPI. A positive value is used as the
     // DPI. This sets the physical size of a device pixel and thus controls the
     // interpretation of physical units.
-    int32_t prefDPI = Preferences::GetInt("layout.css.dpi", -1);
-
+    int32_t prefDPI = StaticPrefs::layout_css_dpi();
     if (prefDPI > 0) {
       dpi = prefDPI;
     } else if (mWidget) {
@@ -317,7 +91,7 @@ void nsDeviceContext::SetDPI(double* aScale) {
       // In case that the widget returns -1, use the primary screen's
       // value as default.
       if (dpi < 0) {
-        primaryScreen->GetDpi(&dpi);
+        dpi = primaryScreen->GetDPI();
       }
       if (prefDPI < 0) {
         dpi = std::max(96.0f, dpi);
@@ -393,6 +167,11 @@ already_AddRefed<gfxContext> nsDeviceContext::CreateRenderingContextCommon(
   MOZ_ASSERT(IsPrinterContext());
   MOZ_ASSERT(mWidth > 0 && mHeight > 0);
 
+  if (NS_WARN_IF(!mPrintTarget)) {
+    // Printing canceled already.
+    return nullptr;
+  }
+
   RefPtr<gfx::DrawTarget> dt;
   if (aWantReferenceContext) {
     dt = mPrintTarget->GetReferenceDrawTarget();
@@ -410,14 +189,6 @@ already_AddRefed<gfxContext> nsDeviceContext::CreateRenderingContextCommon(
     return nullptr;
   }
 
-#ifdef XP_MACOSX
-  // The CGContextRef provided by PMSessionGetCGGraphicsContext is
-  // write-only, so we need to prevent gfxContext::PushGroupAndCopyBackground
-  // trying to read from it or else we'll crash.
-  // XXXjwatt Consider adding a MakeDrawTarget override to PrintTargetCG and
-  // moving this AddUserData call there.
-  dt->AddUserData(&gfxContext::sDontUseAsSourceKey, dt, nullptr);
-#endif
   dt->AddUserData(&sDisablePixelSnapping, (void*)0x1, nullptr);
 
   RefPtr<gfxContext> pContext = gfxContext::CreateOrNull(dt);
@@ -425,20 +196,12 @@ already_AddRefed<gfxContext> nsDeviceContext::CreateRenderingContextCommon(
 
   gfxMatrix transform;
   transform.PreTranslate(mPrintingTranslate);
-  if (mPrintTarget->RotateNeededForLandscape()) {
-    // Rotate page 90 degrees to draw landscape page on portrait paper
-    IntSize size = mPrintTarget->GetSize();
-    transform.PreTranslate(gfxPoint(0, size.width));
-    gfxMatrix rotate(0, -1, 1, 0, 0, 0);
-    transform = rotate * transform;
-  }
   transform.PreScale(mPrintingScale, mPrintingScale);
-
   pContext->SetMatrixDouble(transform);
   return pContext.forget();
 }
 
-nsresult nsDeviceContext::GetDepth(uint32_t& aDepth) {
+uint32_t nsDeviceContext::GetDepth() {
   nsCOMPtr<nsIScreen> screen;
   FindScreen(getter_AddRefs(screen));
   if (!screen) {
@@ -446,9 +209,9 @@ nsresult nsDeviceContext::GetDepth(uint32_t& aDepth) {
     screenManager.GetPrimaryScreen(getter_AddRefs(screen));
     MOZ_ASSERT(screen);
   }
-  screen->GetColorDepth(reinterpret_cast<int32_t*>(&aDepth));
-
-  return NS_OK;
+  int32_t depth = 0;
+  screen->GetColorDepth(&depth);
+  return uint32_t(depth);
 }
 
 nsresult nsDeviceContext::GetDeviceSurfaceDimensions(nscoord& aWidth,
@@ -511,8 +274,8 @@ nsresult nsDeviceContext::InitForPrinting(nsIDeviceContextSpec* aDevice) {
 nsresult nsDeviceContext::BeginDocument(const nsAString& aTitle,
                                         const nsAString& aPrintToFileName,
                                         int32_t aStartPage, int32_t aEndPage) {
-  MOZ_ASSERT(!mIsCurrentlyPrintingDoc,
-             "Mismatched BeginDocument/EndDocument calls");
+  MOZ_DIAGNOSTIC_ASSERT(!mIsCurrentlyPrintingDoc,
+                        "Mismatched BeginDocument/EndDocument calls");
 
   nsresult rv = mPrintTarget->BeginPrinting(aTitle, aPrintToFileName,
                                             aStartPage, aEndPage);
@@ -532,55 +295,64 @@ nsresult nsDeviceContext::BeginDocument(const nsAString& aTitle,
   return rv;
 }
 
-nsresult nsDeviceContext::EndDocument(void) {
-  MOZ_ASSERT(mIsCurrentlyPrintingDoc,
-             "Mismatched BeginDocument/EndDocument calls");
+nsresult nsDeviceContext::EndDocument() {
+  MOZ_DIAGNOSTIC_ASSERT(mIsCurrentlyPrintingDoc,
+                        "Mismatched BeginDocument/EndDocument calls");
+  MOZ_DIAGNOSTIC_ASSERT(mPrintTarget);
 
   mIsCurrentlyPrintingDoc = false;
 
-  nsresult rv = mPrintTarget->EndPrinting();
-  if (NS_SUCCEEDED(rv)) {
+  if (mPrintTarget) {
+    MOZ_TRY(mPrintTarget->EndPrinting());
     mPrintTarget->Finish();
+    mPrintTarget = nullptr;
   }
 
-  if (mDeviceContextSpec) mDeviceContextSpec->EndDocument();
+  if (mDeviceContextSpec) {
+    MOZ_TRY(mDeviceContextSpec->EndDocument());
+  }
 
-  mPrintTarget = nullptr;
-
-  return rv;
+  return NS_OK;
 }
 
-nsresult nsDeviceContext::AbortDocument(void) {
-  MOZ_ASSERT(mIsCurrentlyPrintingDoc,
-             "Mismatched BeginDocument/EndDocument calls");
+nsresult nsDeviceContext::AbortDocument() {
+  MOZ_DIAGNOSTIC_ASSERT(mIsCurrentlyPrintingDoc,
+                        "Mismatched BeginDocument/EndDocument calls");
 
   nsresult rv = mPrintTarget->AbortPrinting();
-
   mIsCurrentlyPrintingDoc = false;
 
-  if (mDeviceContextSpec) mDeviceContextSpec->EndDocument();
+  if (mDeviceContextSpec) {
+    mDeviceContextSpec->EndDocument();
+  }
 
   mPrintTarget = nullptr;
 
   return rv;
 }
 
-nsresult nsDeviceContext::BeginPage(void) {
-  nsresult rv = NS_OK;
-
-  if (mDeviceContextSpec) rv = mDeviceContextSpec->BeginPage();
-
-  if (NS_FAILED(rv)) return rv;
-
-  return mPrintTarget->BeginPage();
+nsresult nsDeviceContext::BeginPage() {
+  MOZ_DIAGNOSTIC_ASSERT(!mIsCurrentlyPrintingDoc || mPrintTarget,
+                        "What nulled out our print target while printing?");
+  if (mDeviceContextSpec) {
+    MOZ_TRY(mDeviceContextSpec->BeginPage());
+  }
+  if (mPrintTarget) {
+    MOZ_TRY(mPrintTarget->BeginPage());
+  }
+  return NS_OK;
 }
 
-nsresult nsDeviceContext::EndPage(void) {
-  nsresult rv = mPrintTarget->EndPage();
-
-  if (mDeviceContextSpec) mDeviceContextSpec->EndPage();
-
-  return rv;
+nsresult nsDeviceContext::EndPage() {
+  MOZ_DIAGNOSTIC_ASSERT(!mIsCurrentlyPrintingDoc || mPrintTarget,
+                        "What nulled out our print target while printing?");
+  if (mPrintTarget) {
+    MOZ_TRY(mPrintTarget->EndPage());
+  }
+  if (mDeviceContextSpec) {
+    MOZ_TRY(mDeviceContextSpec->EndPage());
+  }
+  return NS_OK;
 }
 
 void nsDeviceContext::ComputeClientRectUsingScreen(nsRect* outRect) {
@@ -697,19 +469,3 @@ DesktopToLayoutDeviceScale nsDeviceContext::GetDesktopToDeviceScale() {
 
   return DesktopToLayoutDeviceScale(1.0);
 }
-
-bool nsDeviceContext::IsSyncPagePrinting() const {
-  MOZ_ASSERT(mPrintTarget);
-  return mPrintTarget->IsSyncPagePrinting();
-}
-
-void nsDeviceContext::RegisterPageDoneCallback(
-    PrintTarget::PageDoneCallback&& aCallback) {
-  MOZ_ASSERT(mPrintTarget && aCallback && !IsSyncPagePrinting());
-  mPrintTarget->RegisterPageDoneCallback(std::move(aCallback));
-}
-void nsDeviceContext::UnregisterPageDoneCallback() {
-  if (mPrintTarget) {
-    mPrintTarget->UnregisterPageDoneCallback();
-  }
-}
