@@ -39,16 +39,21 @@ static double WSSDistance(const Face* aFace, const gfxFontStyle& aStyle) {
          weightDist * kWeightFactor;
 }
 
-void* Pointer::ToPtr(FontList* aFontList) const {
+void* Pointer::ToPtr(FontList* aFontList, size_t aSize) const {
   if (IsNull()) {
     return nullptr;
   }
-  uint32_t block = Block();
+
+  // On failure, we'll return null; callers need to handle this appropriately
+  // (e.g. via fallback).
+  void* result = nullptr;
+  uint32_t blockIndex = Block();
+
   // If the Pointer refers to a block we have not yet mapped in this process,
   // we first need to retrieve new block handle(s) from the parent and update
   // our mBlocks list.
   auto& blocks = aFontList->mBlocks;
-  if (block >= blocks.Length()) {
+  if (blockIndex >= blocks.Length()) {
     if (XRE_IsParentProcess()) {
       // Shouldn't happen! A content process tried to pass a bad Pointer?
       return nullptr;
@@ -64,18 +69,27 @@ void* Pointer::ToPtr(FontList* aFontList) const {
     if (!NS_IsMainThread() || !aFontList->UpdateShmBlocks()) {
       return nullptr;
     }
-    MOZ_ASSERT(block < blocks.Length(), "failure in UpdateShmBlocks?");
+    MOZ_ASSERT(blockIndex < blocks.Length(), "failure in UpdateShmBlocks?");
     // This is wallpapering bug 1667977; it's unclear if we will always survive
     // this, as the content process may be unable to shape/render text if all
     // font lookups are failing.
     // In at least some cases, however, this can occur transiently while the
     // font list is being rebuilt by the parent; content will then be notified
     // that the list has changed, and should refresh everything successfully.
-    if (block >= blocks.Length()) {
+    if (blockIndex >= blocks.Length()) {
       return nullptr;
     }
   }
-  return static_cast<char*>(blocks[block]->Memory()) + Offset();
+
+  {
+    // Don't create a pointer that's outside what the block has allocated!
+    const auto& block = blocks[blockIndex];
+    if (Offset() + aSize <= block->Allocated()) {
+      result = static_cast<char*>(block->Memory()) + Offset();
+    }
+  }
+
+  return result;
 }
 
 void String::Assign(const nsACString& aString, FontList* aList) {
@@ -84,7 +98,7 @@ void String::Assign(const nsACString& aString, FontList* aList) {
   MOZ_ASSERT(mPointer.IsNull());
   mLength = aString.Length();
   mPointer = aList->Alloc(mLength + 1);
-  char* p = static_cast<char*>(mPointer.ToPtr(aList));
+  auto* p = mPointer.ToArray<char>(aList, mLength);
   std::memcpy(p, aString.BeginReading(), mLength);
   p[mLength] = '\0';
 }
@@ -196,14 +210,14 @@ void Family::AddFaces(FontList* aList, const nsTArray<Face::InitData>& aFaces) {
   // Allocate space for the face records, and initialize them.
   // coverity[suspicious_sizeof]
   Pointer p = aList->Alloc(count * sizeof(Pointer));
-  auto facePtrs = static_cast<Pointer*>(p.ToPtr(aList));
+  auto* facePtrs = p.ToArray<Pointer>(aList, count);
   for (size_t i = 0; i < count; i++) {
     if (isSimple && !slots[i]) {
       facePtrs[i] = Pointer::Null();
     } else {
       const auto* initData = isSimple ? slots[i] : &aFaces[i];
       Pointer fp = aList->Alloc(sizeof(Face));
-      auto* face = static_cast<Face*>(fp.ToPtr(aList));
+      auto* face = fp.ToPtr<Face>(aList);
       (void)new (face) Face(aList, *initData);
       facePtrs[i] = fp;
       if (initData->mCharMap) {
@@ -253,7 +267,7 @@ bool Family::FindAllFacesForStyleInternal(FontList* aList,
   // whether the size is acceptable.)
   if (NumFaces() == 1) {
     MOZ_ASSERT(!facePtrs[0].IsNull());
-    Face* face = static_cast<Face*>(facePtrs[0].ToPtr(aList));
+    auto* face = facePtrs[0].ToPtr<Face>(aList);
     if (face && face->HasValidDescriptor()) {
       aFaceList.AppendElement(face);
 #ifdef MOZ_WIDGET_GTK
@@ -284,7 +298,7 @@ bool Family::FindAllFacesForStyleInternal(FontList* aList,
         (wantItalic ? kItalicMask : 0) | (wantBold ? kBoldMask : 0);
 
     // If the desired style is available, use it directly.
-    Face* face = static_cast<Face*>(facePtrs[faceIndex].ToPtr(aList));
+    auto* face = facePtrs[faceIndex].ToPtr<Face>(aList);
     if (face && face->HasValidDescriptor()) {
       aFaceList.AppendElement(face);
 #ifdef MOZ_WIDGET_GTK
@@ -309,7 +323,7 @@ bool Family::FindAllFacesForStyleInternal(FontList* aList,
     for (uint8_t trial = 0; trial < 3; ++trial) {
       // check remaining faces in order of preference to find the first that
       // actually exists
-      face = static_cast<Face*>(facePtrs[order[trial]].ToPtr(aList));
+      face = facePtrs[order[trial]].ToPtr<Face>(aList);
       if (face && face->HasValidDescriptor()) {
         aFaceList.AppendElement(face);
 #ifdef MOZ_WIDGET_GTK
@@ -344,7 +358,7 @@ bool Family::FindAllFacesForStyleInternal(FontList* aList,
   // the selected set.
   bool anyNonScalable = false;
   for (uint32_t i = 0; i < NumFaces(); i++) {
-    Face* face = static_cast<Face*>(facePtrs[i].ToPtr(aList));
+    auto* face = facePtrs[i].ToPtr<Face>(aList);
     if (face) {
       // weight/style/stretch priority: stretch >> style >> weight
       double distance = WSSDistance(face, aStyle);
@@ -445,8 +459,7 @@ Face* Family::FindFaceForStyle(FontList* aList, const gfxFontStyle& aStyle,
 
 void Family::SearchAllFontsForChar(FontList* aList,
                                    GlobalFontMatch* aMatchData) {
-  const SharedBitSet* charmap =
-      static_cast<const SharedBitSet*>(mCharacterMap.ToPtr(aList));
+  auto* charmap = mCharacterMap.ToPtr<const SharedBitSet>(aList);
   if (!charmap) {
     // If the face list is not yet initialized, or if character maps have
     // not been loaded, go ahead and do this now (by sending a message to the
@@ -458,7 +471,7 @@ void Family::SearchAllFontsForChar(FontList* aList,
                                                                    true)) {
       return;
     }
-    charmap = static_cast<const SharedBitSet*>(mCharacterMap.ToPtr(aList));
+    charmap = mCharacterMap.ToPtr<const SharedBitSet>(aList);
   }
   if (charmap && !charmap->test(aMatchData->mCh)) {
     return;
@@ -471,14 +484,13 @@ void Family::SearchAllFontsForChar(FontList* aList,
     return;
   }
   for (uint32_t i = 0; i < numFaces; i++) {
-    Face* face = static_cast<Face*>(facePtrs[i].ToPtr(aList));
+    auto* face = facePtrs[i].ToPtr<Face>(aList);
     if (!face) {
       continue;
     }
     MOZ_ASSERT(face->HasValidDescriptor());
     // Get the face's character map, if available (may be null!)
-    charmap =
-        static_cast<const SharedBitSet*>(face->mCharacterMap.ToPtr(aList));
+    charmap = face->mCharacterMap.ToPtr<const SharedBitSet>(aList);
     if (charmap) {
       ++charMapsLoaded;
     }
@@ -536,7 +548,7 @@ void Family::SetFacePtrs(FontList* aList, nsTArray<Pointer>& aFaces) {
     Pointer slots[4] = {Pointer::Null(), Pointer::Null(), Pointer::Null(),
                         Pointer::Null()};
     for (const Pointer& fp : aFaces) {
-      const Face* f = static_cast<const Face*>(fp.ToPtr(aList));
+      auto* f = fp.ToPtr<const Face>(aList);
       if (!f->mWeight.IsSingle() || !f->mStyle.IsSingle() ||
           !f->mStretch.IsSingle()) {
         isSimple = false;
@@ -562,7 +574,7 @@ void Family::SetFacePtrs(FontList* aList, nsTArray<Pointer>& aFaces) {
     if (isSimple) {
       size_t size = 4 * sizeof(Pointer);
       mFaces = aList->Alloc(size);
-      memcpy(mFaces.ToPtr(aList), slots, size);
+      memcpy(mFaces.ToPtr(aList, size), slots, size);
       mFaceCount.store(4);
       mIsSimple = true;
       return;
@@ -570,7 +582,7 @@ void Family::SetFacePtrs(FontList* aList, nsTArray<Pointer>& aFaces) {
   }
   size_t size = aFaces.Length() * sizeof(Pointer);
   mFaces = aList->Alloc(size);
-  memcpy(mFaces.ToPtr(aList), aFaces.Elements(), size);
+  memcpy(mFaces.ToPtr(aList, size), aFaces.Elements(), size);
   mFaceCount.store(aFaces.Length());
 }
 
@@ -589,18 +601,18 @@ void Family::SetupFamilyCharMap(FontList* aList) {
   }
   gfxSparseBitSet familyMap;
   Pointer firstMapShmPointer;
-  SharedBitSet* firstMap = nullptr;
+  const SharedBitSet* firstMap = nullptr;
   bool merged = false;
   Pointer* faces = Faces(aList);
   if (!faces) {
     return;
   }
   for (size_t i = 0; i < NumFaces(); i++) {
-    auto f = static_cast<Face*>(faces[i].ToPtr(aList));
+    auto* f = faces[i].ToPtr<const Face>(aList);
     if (!f) {
       continue;  // Skip missing face (in an incomplete "simple" family)
     }
-    auto faceMap = static_cast<SharedBitSet*>(f->mCharacterMap.ToPtr(aList));
+    auto* faceMap = f->mCharacterMap.ToPtr<const SharedBitSet>(aList);
     if (!faceMap) {
       continue;  // If there's a face where setting up the cmap failed, we skip
                  // it as unusable.
@@ -633,7 +645,7 @@ FontList::FontList(uint32_t aGeneration) {
     // Create the initial shared block, and initialize Header
     if (AppendShmBlock(SHM_BLOCK_SIZE)) {
       Header& header = GetHeader();
-      header.mBlockHeader.mAllocated = sizeof(Header);
+      header.mBlockHeader.mAllocated.store(sizeof(Header));
       header.mGeneration = aGeneration;
       header.mFamilyCount = 0;
       header.mBlockCount.store(1);
@@ -710,7 +722,7 @@ bool FontList::AppendShmBlock(uint32_t aSizeNeeded) {
   }
 
   ShmBlock* block = new ShmBlock(std::move(newShm));
-  block->Allocated() = sizeof(BlockHeader);
+  block->StoreAllocated(sizeof(BlockHeader));
   block->BlockSize() = size;
 
   mBlocks.AppendElement(block);
@@ -887,7 +899,7 @@ Pointer FontList::Alloc(uint32_t aSize) {
   }
 
   // We've found a block; allocate space from it, and return
-  mBlocks[blockIndex]->Allocated() = curAlloc + aSize;
+  mBlocks[blockIndex]->StoreAllocated(curAlloc + aSize);
 
   return Pointer(blockIndex, curAlloc);
 }
@@ -925,7 +937,9 @@ void FontList::SetFamilyNames(nsTArray<Family::InitData>& aFamilies) {
     return;
   }
 
-  Family* families = static_cast<Family*>(header.mFamilies.ToPtr(this));
+  // We can't call Families() here because the mFamilyCount field has not yet
+  // been set!
+  auto* families = header.mFamilies.ToArray<Family>(this, count);
   for (size_t i = 0; i < count; i++) {
     (void)new (&families[i]) Family(this, aFamilies[i]);
     LOG_FONTLIST(("(shared-fontlist) family %u (%s)", (unsigned)i,
@@ -960,7 +974,7 @@ void FontList::SetAliases(
     return;
   }
   fontlist::Pointer ptr = Alloc(count * sizeof(Family));
-  Family* aliases = static_cast<Family*>(ptr.ToPtr(this));
+  auto* aliases = ptr.ToArray<Family>(this, count);
   for (size_t i = 0; i < count; i++) {
     (void)new (&aliases[i]) Family(this, aliasArray[i]);
     LOG_FONTLIST(("(shared-fontlist) alias family %u (%s: %s)", (unsigned)i,
@@ -969,7 +983,7 @@ void FontList::SetAliases(
     if (LOG_FONTLIST_ENABLED()) {
       const auto& faces = aAliasTable.Get(aliasArray[i].mKey)->mFaces;
       for (unsigned j = 0; j < faces.Length(); j++) {
-        auto face = static_cast<const fontlist::Face*>(faces[j].ToPtr(this));
+        auto* face = faces[j].ToPtr<const Face>(this);
         const nsCString& desc = face->mDescriptor.AsString(this);
         nsAutoCString weight, style, stretch;
         face->mWeight.ToString(weight);
@@ -1002,7 +1016,7 @@ void FontList::SetLocalNames(
   size_t count = faceArray.Length();
   Family* families = Families();
   fontlist::Pointer ptr = Alloc(count * sizeof(LocalFaceRec));
-  LocalFaceRec* faces = static_cast<LocalFaceRec*>(ptr.ToPtr(this));
+  auto* faces = ptr.ToArray<LocalFaceRec>(this, count);
   for (size_t i = 0; i < count; i++) {
     (void)new (&faces[i]) LocalFaceRec();
     const auto& rec = aLocalNameTable.Get(faceArray[i]);
@@ -1023,7 +1037,7 @@ void FontList::SetLocalNames(
           static_cast<const Pointer*>(family->Faces(this));
       for (uint32_t j = 0; j < family->NumFaces(); j++) {
         if (!faceList[j].IsNull()) {
-          const Face* f = static_cast<const Face*>(faceList[j].ToPtr(this));
+          auto* f = faceList[j].ToPtr<const Face>(this);
           if (f && rec.mFaceDescriptor == f->mDescriptor.AsString(this)) {
             faces[i].mFaceIndex = j;
             break;
@@ -1079,7 +1093,7 @@ Family* FontList::FindFamily(const nsCString& aName, bool aPrimaryNameOnly) {
     const nsCString& mTarget;
   };
 
-  Header& header = GetHeader();
+  const Header& header = GetHeader();
 
   Family* families = Families();
   if (!families) {
@@ -1230,7 +1244,7 @@ void FontList::SearchForLocalFace(const nsACString& aName, Family** aFamily,
       continue;
     }
     for (uint32_t j = 0; j < family->NumFaces(); j++) {
-      Face* face = static_cast<Face*>(faces[j].ToPtr(this));
+      auto* face = faces[j].ToPtr<Face>(this);
       if (!face) {
         continue;
       }
